#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>,
#
# Version: 1.0

import logging
import traceback
import requests
from io import BytesIO
try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse

from Ice import Exception as IceException
from omero import ApiUsageException, ServerError

from django.conf import settings
from .mapr_settings import mapr_settings

from django.core.urlresolvers import reverse
from django.http import HttpResponseServerError, HttpResponseBadRequest
from django.http import JsonResponse
from django.http import Http404

from django.core.validators import URLValidator
from django.core.exceptions import ValidationError

from django.utils.html import strip_tags

from django_redis import get_redis_connection

from omero.gateway.utils import toBoolean

from .show import mapr_paths_to_object
from .show import MapShow as Show
from .tree import count_mapannotations, \
                  marshal_mapannotations, \
                  marshal_screens, \
                  marshal_projects, \
                  marshal_datasets, \
                  marshal_plates, \
                  marshal_images, \
                  load_mapannotation, \
                  marshal_autocomplete

from omeroweb.webclient.decorators import login_required, render_response
from omeroweb.webclient.views import get_long_or_default, get_bool_or_default

from omeroweb.webclient import tree as webclient_tree
from omeroweb.webclient.views import _load_template as _webclient_load_template

from omeroweb.webclient.views import api_paths_to_object \
    as webclient_api_paths_to_object

try:
    # renamed for Django 1.11
    from omeroweb.httprsp import HttpJPEGResponse
except ImportError:
    # old name for backwards compatibility
    from omeroweb.http import HttpJPEGResponse

import omeroweb

logger = logging.getLogger(__name__)


try:
    from PIL import Image  # see ticket:2597
except ImportError:
    try:
        import Image  # see ticket:2597
    except ImportError:
        logger.error(
            "You need to install the Python Imaging Library. Get it at"
            " http://www.pythonware.com/products/pil/")
        logger.error(traceback.format_exc())


# Views Helpers
def fake_experimenter(label):
    """
    Marshal faked experimenter when id is -1
    Load omero.client.ui.menu.dropdown.everyone.label as username
    """
    return {
        'id': -1,
        'omeName': label,
        'firstName': label,
        'lastName': ''
    }


def get_unicode_or_default(request, name, default):
    """
    Retrieves a parameter from the request. If the parameter is not present
    the default is returned

    This does not catch exceptions as it makes sense to throw exceptions if
    the arguments provided do not pass basic type validation
    """
    val = None
    val_raw = request.GET.get(name, default)
    if val_raw is not None:
        val = strip_tags(val_raw)
    return val


def _get_wildcard(mapr_settings, menu):
    wc = False
    try:
        wc = mapr_settings.CONFIG[menu]['wildcard']['enabled']
    except KeyError:
        pass
    return wc


def _get_wildcard_limit(mapr_settings, menu):
    wc = 0
    try:
        wc = mapr_settings.CONFIG[menu]['wildcard']['limit']
    except KeyError:
        pass
    return wc


def _get_ns(mapr_settings, menu):
    ns = []
    try:
        ns = mapr_settings.CONFIG[menu]['ns']
    except KeyError:
        pass
    return ns


def _get_keys(mapr_settings, menu):
    keys = None
    try:
        keys = mapr_settings.CONFIG[menu]['all']
    except KeyError:
        pass
    return keys


def _get_case_sensitive(mapr_settings, menu):
    cs = False
    try:
        cs = toBoolean(mapr_settings.CONFIG[menu]['case_sensitive'])
    except KeyError:
        pass
    return cs


def _get_page(request):
    page = get_long_or_default(request, 'page', 1)
    if page < 1:
        page = 1
    return page


@login_required()
@render_response()
def index(request, menu, conn=None, url=None, **kwargs):
    """
    This override omeroweb.webclient.load_template
    to support custom template base_mapr.html
    """

    try:
        value = get_unicode_or_default(request, 'value', None)
        query = get_bool_or_default(request, 'query', False)
        if _get_case_sensitive(mapr_settings, menu):
            case_sensitive = get_bool_or_default(
                request, 'case_sensitive', False)
        else:
            case_sensitive = False
    except ValueError:
        logger.error(traceback.format_exc())
        return HttpResponseBadRequest('Invalid parameter value')
    kwargs['show'] = Show(conn=conn, request=request, menu=menu, value=value)
    kwargs['load_template_url'] = reverse(viewname="maprindex_%s" % menu)
    kwargs['template'] = "mapr/base_mapr.html"
    context = _webclient_load_template(request, menu,
                                       conn=conn, url=url, **kwargs)
    context['active_user'] = context.get('active_user', {'id': -1})
    context['mapr_conf'] = {
        'menu': menu,
        'menu_all': mapr_settings.CONFIG[menu]['all'],
        'menu_default': mapr_settings.CONFIG[menu]['default'],
        'case_sensitive': _get_case_sensitive(mapr_settings, menu)}
    context['map_ctx'] = \
        {'label': menu, 'value': value or "", 'query': query or "",
         'case_sensitive': case_sensitive or ""}
    context['template'] = "mapr/base_mapr.html"

    return context


def api_mapr_config(request):
    """Return mapr_settings.CONFIG as JSON."""
    return JsonResponse(mapr_settings.CONFIG)


@login_required()
def api_paths_to_object(request, menu=None, conn=None, **kwargs):
    """
    This override omeroweb.webclient.api_paths_to_object
    to support custom path to map.value
    """

    try:
        mapann_ns = _get_ns(mapr_settings, menu)
        mapann_names = _get_keys(mapr_settings, menu)

        mapann_value = get_unicode_or_default(request, 'map.value', None)
    except ValueError:
        return HttpResponseBadRequest('Invalid parameter value')

    if menu in mapr_settings.CONFIG and mapann_value:
        paths = []
        try:
            experimenter_id = get_long_or_default(request, 'experimenter',
                                                  None)
            project_id = get_long_or_default(request, 'project', None)
            dataset_id = get_long_or_default(request, 'dataset', None)
            image_id = get_long_or_default(request, 'image', None)
            screen_id = get_long_or_default(request, 'screen', None)
            plate_id = get_long_or_default(request, 'plate', None)
            group_id = get_long_or_default(request, 'group', None)
        except ValueError:
            return HttpResponseBadRequest('Invalid parameter value')

        paths = mapr_paths_to_object(
            conn=conn, mapann_value=mapann_value,
            mapann_ns=mapann_ns, mapann_names=mapann_names,
            screen_id=screen_id, plate_id=plate_id,
            project_id=project_id, dataset_id=dataset_id,
            image_id=image_id,
            experimenter_id=experimenter_id, group_id=group_id)

        return JsonResponse({'paths': paths})
    return webclient_api_paths_to_object(request, conn=conn, **kwargs)


omeroweb.webclient.views.api_paths_to_object = api_paths_to_object


@login_required()
def api_experimenter_list(request, menu, conn=None, **kwargs):

    # Get parameters
    try:
        mapann_ns = _get_ns(mapr_settings, menu)
        mapann_names = _get_keys(mapr_settings, menu)

        # page = _get_page(request)
        # limit = get_long_or_default(request, 'limit', settings.PAGE)
        group_id = get_long_or_default(request, 'group', -1)
        experimenter_id = get_long_or_default(request, 'experimenter', -1)
        mapann_value = get_unicode_or_default(request, 'value', None) \
            or get_unicode_or_default(request, 'id', None)
        query = get_bool_or_default(request, 'query', False)
        if _get_case_sensitive(mapr_settings, menu):
            case_sensitive = get_bool_or_default(
                request, 'case_sensitive', False)
        else:
            case_sensitive = False
    except ValueError:
        return HttpResponseBadRequest('Invalid parameter value')

    experimenter = {}
    try:
        if experimenter_id > -1:
            # Get the experimenter
            experimenter = webclient_tree.marshal_experimenter(
                conn=conn, experimenter_id=experimenter_id)
        else:
            # fake experimenter -1
            experimenter = fake_experimenter(
                mapr_settings.CONFIG[menu]['label'])

        if _get_wildcard(mapr_settings, menu) or mapann_value:
            experimenter['extra'] = {'case_sensitive': case_sensitive}
            if query:
                experimenter['extra']['query'] = query

            # count children
            experimenter['childCount'] = count_mapannotations(
                conn=conn,
                mapann_value=mapann_value,
                query=query,
                case_sensitive=case_sensitive,
                mapann_ns=mapann_ns,
                mapann_names=mapann_names,
                group_id=group_id,
                experimenter_id=experimenter_id)

            if experimenter['childCount'] > 0 and mapann_value:
                experimenter['extra']['value'] = mapann_value

    except ApiUsageException as e:
        return HttpResponseBadRequest(e.serverStackTrace)
    except ServerError as e:
        return HttpResponseServerError(e.serverStackTrace)
    except IceException as e:
        return HttpResponseServerError(e.message)

    return JsonResponse({'experimenter': experimenter})


@login_required()
def api_mapannotation_list(request, menu, conn=None, **kwargs):

    # Get parameters
    try:
        mapann_ns = _get_ns(mapr_settings, menu)
        mapann_names = _get_keys(mapr_settings, menu)

        page = page = _get_page(request)
        limit = get_long_or_default(request, 'limit', settings.PAGE)
        group_id = get_long_or_default(request, 'group', -1)
        experimenter_id = get_long_or_default(request, 'experimenter_id', -1)
        mapann_value = get_unicode_or_default(request, 'id', None) \
            or get_unicode_or_default(request, 'value', None)
        query = get_bool_or_default(request, 'query', False)
        if _get_case_sensitive(mapr_settings, menu):
            case_sensitive = get_bool_or_default(
                request, 'case_sensitive', False)
        else:
            case_sensitive = False
        orphaned = get_bool_or_default(request, 'orphaned', False)
    except ValueError:
        logger.error(traceback.format_exc())
        return HttpResponseBadRequest('Invalid parameter value')

    mapannotations = []
    screens = []
    projects = []
    try:
        if _get_wildcard(mapr_settings, menu) or mapann_value:
            # Get attributes from map annotation
            if orphaned:
                # offset = _get_wildcard_limit(mapr_settings, menu)
                mapannotations = marshal_mapannotations(
                    conn=conn,
                    mapann_value=mapann_value,
                    query=query,
                    case_sensitive=case_sensitive,
                    mapann_ns=mapann_ns,
                    mapann_names=mapann_names,
                    group_id=group_id,
                    experimenter_id=experimenter_id,
                    page=page,
                    limit=limit)
            else:
                screens = marshal_screens(
                    conn=conn,
                    mapann_value=mapann_value,
                    query=query,
                    mapann_ns=mapann_ns,
                    mapann_names=mapann_names,
                    group_id=group_id,
                    experimenter_id=experimenter_id,
                    page=page,
                    limit=limit)
                projects = marshal_projects(
                    conn=conn,
                    mapann_value=mapann_value,
                    query=query,
                    mapann_ns=mapann_ns,
                    mapann_names=mapann_names,
                    group_id=group_id,
                    experimenter_id=experimenter_id,
                    page=page,
                    limit=limit)

    except ApiUsageException as e:
        return HttpResponseBadRequest(e.serverStackTrace)
    except ServerError as e:
        return HttpResponseServerError(e.serverStackTrace)
    except IceException as e:
        return HttpResponseServerError(e.message)

    return JsonResponse({'maps': mapannotations,
                         'screens': screens, 'projects': projects})


@login_required()
def api_datasets_list(request, menu, conn=None, **kwargs):

    # Get parameters
    try:
        mapann_ns = _get_ns(mapr_settings, menu)
        mapann_names = _get_keys(mapr_settings, menu)

        page = _get_page(request)
        limit = get_long_or_default(request, 'limit', settings.PAGE)
        group_id = get_long_or_default(request, 'group', -1)
        experimenter_id = get_long_or_default(request,
                                              'experimenter_id', -1)
        project_id = get_long_or_default(request, 'id', None)
        mapann_value = get_unicode_or_default(request, 'value', None)
        query = get_bool_or_default(request, 'query', False)
    except ValueError:
        return HttpResponseBadRequest('Invalid parameter value')

    datasets = []
    try:
        if _get_wildcard(mapr_settings, menu) or mapann_value:
            # Get the images
            datasets = marshal_datasets(
                conn=conn,
                project_id=project_id,
                mapann_value=mapann_value,
                query=query,
                mapann_ns=mapann_ns,
                mapann_names=mapann_names,
                group_id=group_id,
                experimenter_id=experimenter_id,
                page=page,
                limit=limit)
    except ApiUsageException as e:
        return HttpResponseBadRequest(e.serverStackTrace)
    except ServerError as e:
        return HttpResponseServerError(e.serverStackTrace)
    except IceException as e:
        return HttpResponseServerError(e.message)

    return JsonResponse({'datasets': datasets})


@login_required()
def api_plate_list(request, menu, conn=None, **kwargs):

    # Get parameters
    try:
        mapann_ns = _get_ns(mapr_settings, menu)
        mapann_names = _get_keys(mapr_settings, menu)

        page = _get_page(request)
        limit = get_long_or_default(request, 'limit', settings.PAGE)
        group_id = get_long_or_default(request, 'group', -1)
        experimenter_id = get_long_or_default(request,
                                              'experimenter_id', -1)
        screen_id = get_long_or_default(request, 'id', None)
        mapann_value = get_unicode_or_default(request, 'value', None)
        query = get_bool_or_default(request, 'query', False)
    except ValueError:
        return HttpResponseBadRequest('Invalid parameter value')

    plates = []
    try:
        if _get_wildcard(mapr_settings, menu) or mapann_value:
            # Get the images
            plates = marshal_plates(
                conn=conn,
                screen_id=screen_id,
                mapann_value=mapann_value,
                query=query,
                mapann_ns=mapann_ns,
                mapann_names=mapann_names,
                group_id=group_id,
                experimenter_id=experimenter_id,
                page=page,
                limit=limit)
    except ApiUsageException as e:
        return HttpResponseBadRequest(e.serverStackTrace)
    except ServerError as e:
        return HttpResponseServerError(e.serverStackTrace)
    except IceException as e:
        return HttpResponseServerError(e.message)

    return JsonResponse({'plates': plates})


@login_required()
def api_image_list(request, menu, conn=None, **kwargs):

    # Get parameters
    try:
        mapann_ns = _get_ns(mapr_settings, menu)
        mapann_names = _get_keys(mapr_settings, menu)

        page = _get_page(request)
        limit = get_long_or_default(request, 'limit', settings.PAGE)
        group_id = get_long_or_default(request, 'group', -1)
        load_pixels = get_bool_or_default(request, 'sizeXYZ', False)
        thumb_version = get_bool_or_default(request, 'thumbVersion', False)
        date = get_bool_or_default(request, 'date', False)
        experimenter_id = get_long_or_default(request,
                                              'experimenter_id', -1)
        parent = get_unicode_or_default(request, 'node', None)
        parent_id = get_long_or_default(request, 'id', None)
        mapann_value = get_unicode_or_default(request, 'value', None)
        query = get_bool_or_default(request, 'query', False)
    except ValueError:
        return HttpResponseBadRequest('Invalid parameter value')

    images = []
    try:
        if _get_wildcard(mapr_settings, menu) or mapann_value:
            # Get the images
            images = marshal_images(
                conn=conn,
                parent=parent,
                parent_id=parent_id,
                mapann_ns=mapann_ns,
                mapann_names=mapann_names,
                mapann_value=mapann_value,
                query=query,
                load_pixels=load_pixels,
                group_id=group_id,
                experimenter_id=experimenter_id,
                page=page,
                date=date,
                thumb_version=thumb_version,
                limit=limit)
    except ApiUsageException as e:
        return HttpResponseBadRequest(e.serverStackTrace)
    except ServerError as e:
        return HttpResponseServerError(e.serverStackTrace)
    except IceException as e:
        return HttpResponseServerError(e.message)

    return JsonResponse({'images': images})


@login_required()
@render_response()
def load_metadata_details(request, c_type, conn=None, share_id=None,
                          **kwargs):
    """
    This page is the right-hand panel 'general metadata', first tab only.
    Shown for Projects, Datasets, Images, Screens, Plates, Wells, Tags etc.
    The data and annotations are loaded by the manager. Display of appropriate
    data is handled by the template.
    """

    c_id = None
    try:
        c_id = get_unicode_or_default(request, 'value', None)
    except ValueError:
        return HttpResponseBadRequest('Invalid parameter value')

    template = "mapr/metadata_general.html"

    context = dict()
    context['template'] = template
    context['menu'] = c_type
    context['manager'] = {'obj_type': 'map', 'obj_id': c_id}
    context['maprindex'] = reverse(viewname=('maprindex'))
    context['maprindex_path'] = reverse(viewname=('maprindex_%s' % c_type))
    return context


@login_required()
def api_annotations(request, menu, conn=None, **kwargs):

    # Get parameters
    try:
        mapann_ns = _get_ns(mapr_settings, menu)

        mapann_value = get_unicode_or_default(request, 'map', None)
        mapann_names = _get_keys(mapr_settings, menu)
    except ValueError:
        return HttpResponseBadRequest('Invalid parameter value')

    anns = []
    exps = []
    try:
        anns, exps = load_mapannotation(
            conn=conn,
            mapann_ns=mapann_ns,
            mapann_names=mapann_names,
            mapann_value=mapann_value)
    except ApiUsageException as e:
        return HttpResponseBadRequest(e.serverStackTrace)
    except ServerError as e:
        return HttpResponseServerError(e.serverStackTrace)
    except IceException as e:
        return HttpResponseServerError(e.message)

    return JsonResponse({'annotations': anns, 'experimenters': exps})


@login_required()
def mapannotations_autocomplete(request, menu, conn=None, **kwargs):

    # Get parameters
    try:
        mapann_ns = _get_ns(mapr_settings, menu)
        mapann_names = _get_keys(mapr_settings, menu)

        page = _get_page(request)
        limit = get_long_or_default(request, 'limit', settings.PAGE)
        group_id = get_long_or_default(request, 'group', -1)
        experimenter_id = get_long_or_default(request, 'experimenter_id', -1)
        mapann_value = get_unicode_or_default(request, 'value', None)
        query = get_bool_or_default(request, 'query', True)
        if _get_case_sensitive(mapr_settings, menu):
            case_sensitive = get_bool_or_default(
                request, 'case_sensitive', False)
        else:
            case_sensitive = False
    except ValueError:
        return HttpResponseBadRequest('Invalid parameter value')

    autocomplete = []
    try:
        if mapann_value:
            autocomplete = marshal_autocomplete(
                conn=conn,
                mapann_value=mapann_value,
                query=query,
                case_sensitive=case_sensitive,
                mapann_ns=mapann_ns,
                mapann_names=mapann_names,
                group_id=group_id,
                experimenter_id=experimenter_id,
                page=page,
                limit=limit)
    except ApiUsageException as e:
        return HttpResponseBadRequest(e.serverStackTrace)
    except ServerError as e:
        return HttpResponseServerError(e.serverStackTrace)
    except IceException as e:
        return HttpResponseServerError(e.message)

    return JsonResponse(list(autocomplete), safe=False)


@login_required()
def mapannotations_favicon(request, conn=None, **kwargs):

    icon = None
    favdomain = "{0.scheme}://{0.netloc}/".format(
        urlparse(request.GET.get('u', None)))
    if favdomain is not None:
        validate = URLValidator()
        try:
            validate(favdomain)
        except ValidationError:
            return HttpResponseBadRequest('Invalid url')

        _cache_key = "favicon.%s" % favdomain

        cache = get_redis_connection("default")
        icon = cache.hget('favdomain', _cache_key)
        if icon is None:
            try:
                r = requests.get(
                    "%s%s" % (mapr_settings.FAVICON_WEBSERVICE, favdomain),
                    stream=True)
                if r.status_code == 200:
                    icon = r.content
                    cache.hset('favdomain', _cache_key, icon)
            finally:
                r.connection.close()
        return HttpJPEGResponse(icon)

    with Image.open(mapr_settings.DEFAULT_FAVICON) as img:
        img.thumbnail((16, 16), Image.ANTIALIAS)
        f = BytesIO()
        img.save(f, "PNG")
        f.seek(0)
        return HttpJPEGResponse(f.read())
    raise Http404



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>,
#
# Version: 1.0

import sys
import os

from django.conf import settings
from omeroweb.settings import process_custom_settings, report_settings
from omero_mapr.utils import config_list_to_dict


# load settings
MAPR_SETTINGS_MAPPING = {
    "omero.web.mapr.config":
        ["MAPR_CONFIG", "[]", config_list_to_dict, None],
    "omero.web.mapr.favicon":
        ["MAPR_DEFAULT_FAVICON",
         os.path.join(os.path.dirname(__file__),
                      'static', 'mapr', 'image',
                      'favicon.png').replace('\\', '/'),
         str, None],
    "omero.web.mapr.favicon_webservice":
        ["MAPR_FAVICON_WEBSERVICE",
            "http://www.google.com/s2/favicons?domain=", str,
            (
                "Service to load favicons for use in external links."
                " Icons are cached in redis which must be available."
            )
         ],
    }


process_custom_settings(sys.modules[__name__], 'MAPR_SETTINGS_MAPPING')
report_settings(sys.modules[__name__])


def prefix_setting(suffix, default):
    @property
    def func(self):
        return getattr(settings, 'MAPR_%s' % suffix, default)
    return func


class MaprSettings(object):

    CONFIG = prefix_setting('CONFIG', MAPR_CONFIG)  # noqa
    DEFAULT_FAVICON = prefix_setting('DEFAULT_FAVICON',
                                     MAPR_DEFAULT_FAVICON)  # noqa
    FAVICON_WEBSERVICE = prefix_setting('FAVICON_WEBSERVICE',
                                        MAPR_FAVICON_WEBSERVICE)  # noqa


mapr_settings = MaprSettings()


# Update Django settings

# add custom processor
TEMPLATES = getattr(settings, 'TEMPLATES', [])
TEMPLATES[0]['OPTIONS']['context_processors'].append(
    'omero_mapr.custom_context_processor.mapr_url_suffix')
setattr(settings, 'TEMPLATES', TEMPLATES)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#
#
# Copyright (c) 2016 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>.
#
# Version: 1.0
#

from omero.model import ScreenI
from omero.rtypes import rstring, unwrap
from omero.constants.namespaces import NSBULKANNOTATIONS
from omero.util.populate_metadata import BulkToMapAnnotationContext
from omero.util.populate_metadata import ParsingContext

from omeroweb.testlib import IWebTest


class IMaprTest(IWebTest):

    """
    Extends IWebTest (ITest)
    """

    def set_name(self, obj, name):
        q = self.client.sf.getQueryService()
        up = self.client.sf.getUpdateService()
        obj = q.get(obj.__class__.__name__, obj.id.val)
        obj.setName(rstring(name))
        return up.saveAndReturnObject(obj)

    def create_screen(self, row_count, col_count):
        # TODO: remove 5.2 vs 5.3 compatibility
        try:
            plate = self.importPlates(plateRows=row_count,
                                      plateCols=col_count)[0]
        except AttributeError:
            plate = self.import_plates(plate_rows=row_count,
                                       plate_cols=col_count)[0]
        plate = self.set_name(plate, "Plate001")
        screen = ScreenI()
        screen.name = rstring("Screen001")
        screen.linkPlate(plate.proxy())
        return (self.client.sf.getUpdateService().saveAndReturnObject(screen),
                plate)

    def new_screen(self, name=None, description=None):
        """
        Creates a new screen object.
        :param name: The screen name. If None, a UUID string will be used
        :param description: The screen description
        """
        return self.new_object(ScreenI, name=name, description=description)

    def make_screen(self, name=None, description=None, client=None):
        """
        Creates a new screen instance and returns the persisted object.
        :param name: The screen name. If None, a UUID string will be used
        :param description: The screen description
        :param client: The client to use to create the object
        """
        if client is None:
            client = self.client
        screen = self.new_screen(name=name, description=description)
        return client.sf.getUpdateService().saveAndReturnObject(screen)

    def get_screen_annotations(self):
        query = """select s from Screen s
            left outer join fetch s.annotationLinks links
            left outer join fetch links.child
            where s.id=%s""" % self.screen.id.val
        qs = self.client.sf.getQueryService()
        screen = qs.findByQuery(query, None)
        anns = screen.linkedAnnotationList()
        return anns

    def populate_data(self, csv, cfg):
        row_count = 1
        col_count = 6
        self.screen, self.plate = self.create_screen(row_count, col_count)

        ctx = ParsingContext(self.client, self.screen.proxy(), file=csv)
        ctx.parse()
        ctx.write_to_omero()

        # Get file annotations
        anns = self.get_screen_annotations()
        # Only expect a single annotation which is a 'bulk annotation'
        assert len(anns) == 1
        table_file_ann = anns[0]
        assert unwrap(table_file_ann.getNs()) == NSBULKANNOTATIONS
        fileid = table_file_ann.file.id.val

        ctx = BulkToMapAnnotationContext(
            self.client, self.screen.proxy(), fileid=fileid, cfg=cfg)
        ctx.parse()
        ctx.write_to_omero()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 University of Dundee.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Aleksandra Tarkowska <A(dot)Tarkowska(at)dundee(dot)ac(dot)uk>,
#
# Version: 1.0

import logging
import omero

from copy import deepcopy

from django.conf import settings
from .mapr_settings import mapr_settings

from omero.rtypes import rint, rlong, unwrap

import omeroweb.webclient.show as omeroweb_show
from .tree import _set_parameters

from omeroweb.utils import reverse_with_params


logger = logging.getLogger(__name__)


class MapShow(omeroweb_show.Show):

    omeroweb_show.Show.TOP_LEVEL_PREFIXES += ('map',)

    # supported objects
    omeroweb_show.Show.SUPPORTED_OBJECT_TYPES += ('map',)
    omeroweb_show.Show.SUPPORTED_OBJECT_TYPES += \
        tuple(mapr_settings.CONFIG)

    def __init__(self, conn, request, menu, value=None):
        super(MapShow, self).__init__(conn, request, menu)
        if value and len(self._initially_select) < 1:
            path = "map.value-%s" % value
            self._add_if_supported(path)

    def _find_first_selected(self):
        if len(self._initially_select) == 0:
            return None

        # tree hierarchy open to first selected object
        m = self.PATH_REGEX.match(self._initially_select[0])
        if m is None:
            return None
        first_obj = m.group('object_type')
        # if we are showing any of map.value alias make sure
        # we are not in webclient
        if (first_obj in mapr_settings.CONFIG.keys() and
                self.menu not in mapr_settings.CONFIG):
            # redirect to menu/?value=VALUE
            link = {
                "viewname": "maprindex_%s" % first_obj,
                "query_string": {'value': m.group('value')}
            }
            raise omeroweb_show.IncorrectMenuError(
                reverse_with_params(**link)
            )
        # if in mapr app hierachy is different
        if self.menu in mapr_settings.CONFIG:
            first_selected = None
            try:
                key = m.group('key')
                value = m.group('value')
                if key == 'id':
                    value = int(value)
                attributes = {key: value}
                # Set context to 'cross-group'
                self.conn.SERVICE_OPTS.setOmeroGroup('-1')
                first_selected = self._load_first_selected(
                    first_obj, attributes)
            except Exception:
                pass
            if first_obj not in self.TOP_LEVEL_PREFIXES:
                # Need to see if first item has parents
                if first_selected is not None:
                    for p in first_selected.getAncestry():
                        # # we display images directly
                        self._initially_open.insert(
                            0,
                            "%s-%s" % (p.OMERO_CLASS.lower(), p.getId())
                        )
                        self._initially_open_owner = p.details.owner.id.val
                    m = self.PATH_REGEX.match(self._initially_open[0])
                    if m.group('object_type') == 'image':
                        self._initially_open.insert(0, "orphaned-0")
            return first_selected
        return super(MapShow, self)._find_first_selected()

    def _load_first_selected(self, first_obj, attributes):
        first_selected = None
        if first_obj in ["map"]:
            first_selected = self._load_mapannotations(attributes)
            self._initially_open_owner = first_selected.details.owner.id.val
        else:
            first_selected = super(MapShow, self) \
                ._load_first_selected(first_obj, attributes)
        return first_selected

    def _load_mapannotations(self, attributes):
        """
        TODO: make sure it is calling right context (groups, users)
        """
        if 'value' in attributes:
            service_opts = deepcopy(self.conn.SERVICE_OPTS)
            params = omero.sys.ParametersI()
            params.addString("mvalue", attributes['value'])
            f = omero.sys.Filter()
            f.limit = rint(1)
            params.theFilter = f

            q = ("SELECT distinct (a) "
                 "FROM ImageAnnotationLink ial JOIN ial.child a "
                 "JOIN a.mapValue mv "
                 "WHERE mv.value = :mvalue")

            qs = self.conn.getQueryService()
            logger.debug("HQL QUERY: %s\nPARAMS: %r" % (q, params))
            m = qs.findByQuery(q, params, service_opts)
            # hardcode to always tell to load all users
            return omero.gateway.MapAnnotationWrapper(self.conn, m)


omeroweb_show.Show = MapShow


def mapr_paths_to_object(conn, mapann_value,
                         mapann_ns=[], mapann_names=[],
                         screen_id=None, plate_id=None,
                         project_id=None, dataset_id=None,
                         image_id=None,
                         experimenter_id=None, group_id=None,
                         page_size=None, limit=settings.PAGE):

    qs = conn.getQueryService()
    params, where_clause = _set_parameters(
        mapann_ns=mapann_ns, mapann_names=mapann_names,
        query=False, mapann_value=mapann_value,
        params=None, experimenter_id=experimenter_id,
        page=page_size, limit=limit)

    service_opts = deepcopy(conn.SERVICE_OPTS)

    if group_id is not None:
        service_opts.setOmeroGroup(group_id)

    q = []
    q.append(" SELECT distinct new map( "
             " mv.value as map_value, "
             " i.details.owner.id as owner, ")

    q_select = []
    if screen_id or plate_id:
        q_select.append(" sl.parent.id as screen_id, ")
        if plate_id:
            q_select.append(" pl.id as plate_id, ")
    elif project_id or dataset_id:
        q_select.append(" pdl.parent.id as project_id,  ")
        if dataset_id:
            q_select.append(" ds.id as dataset_id, ")
    elif image_id:
        q_select.append(" COALESCE(sl.parent.id,null) as screen_id, "
                        " COALESCE(pdl.parent.id,null) as project_id, "
                        " COALESCE(pl.id,null) as plate_id, "
                        " COALESCE(ds.id,null) as dataset_id "
                        " i.id as image_id, ")

    if q_select:
        q.append(" ".join(q_select))

    q.append(" count(i.id) as imgCount) "
             " FROM ImageAnnotationLink ial "
             " join ial.child a "
             " join a.mapValue mv "
             " join ial.parent i "
             " left outer join i.details.owner "
             " left outer join i.wellSamples ws "
             " left outer join ws.well w "
             " left outer join w.plate pl "
             " left outer join pl.screenLinks sl "
             " left outer join i.datasetLinks dil "
             " left outer join dil.parent ds "
             " left outer join ds.projectLinks pdl "
             " WHERE ( "
             " (dil is null "
             " and ds is null and pdl is null "
             " and ws is not null "
             " and w is not null and pl is not null "
             " and sl is not null) "
             " OR "
             " (ws is null "
             " and w is null and pl is null and sl is null "
             " and dil is not null "
             " and ds is not null and pdl is not null ) "
             " ) "
             " AND ")

    if image_id:
        q.append(" i.id = :iid and ")
        params.add('iid', rlong(image_id))
    elif screen_id:
        q.append(" sl.parent.id = :sid and ")
        params.add('sid', rlong(screen_id))
    elif plate_id:
        q.append(" pl.id = :pid and ")
        params.add('pid', rlong(plate_id))
    elif project_id:
        q.append(" pdl.parent.id = :pid and ")
        params.add('pid', rlong(project_id))
    elif dataset_id:
        q.append(" ds.id = :did and ")
        params.add('did', rlong(dataset_id))

    if len(where_clause) > 0:
        q.append(" and ".join(where_clause))

    q.append(" group by mv.value, i.details.owner.id ")

    q_groupby = []
    if screen_id:
        q_groupby.append(" sl.parent.id ")
    elif plate_id:
        q_groupby.append(" sl.parent.id ")
        q_groupby.append(" pl.id ")
    elif project_id:
        q_groupby.append(" pdl.parent.id ")
    elif dataset_id:
        q_groupby.append(" pdl.parent.id ")
        q_groupby.append(" ds.id ")
    elif image_id:
        q_groupby.append(" COALESCE(sl.parent.id,null) ")
        q_groupby.append(" COALESCE(pdl.parent.id,null) ")
        q_groupby.append(" COALESCE(pl.id,null) ")
        q_groupby.append(" COALESCE(ds.id,null) ")
        q_groupby.append(" i.id ")

    if q_groupby:
        q.append(", " + ", ".join(q_groupby))

    query = "".join(q)

    # Hierarchies for this object
    paths = []

    logger.debug("HQL QUERY: %s\nPARAMS: %r" % (query, params))
    for e in unwrap(qs.projection(query, params, service_opts)):
        path = []

        # Experimenter is always present
        try:
            experimenter_id = e[0]["owner"]
        except Exception:
            experimenter_id = -1
        path.append({
            'type': 'experimenter',
            'id': experimenter_id
        })

        try:
            map_value = e[0]["map_value"]
            path.append({
                'type': 'map',
                'id': map_value,
            })
        except Exception:
            pass

        try:
            if e[0]["screen_id"] is not None:
                path.append({
                    'type': 'screen',
                    'id': e[0]["screen_id"]
                })
        except Exception:
            pass

        try:
            if e[0]["plate_id"] is not None:
                plate_id = e[0]["plate_id"]
                path.append({
                    'type': 'plate',
                    'id': plate_id,
                })
        except Exception:
            pass

        try:
            if e[0]["project_id"] is not None:
                path.append({
                    'type': 'project',
                    'id': e[0]["project_id"]
                })
        except Exception:
            pass

        try:
            if e[0]["dataset_id"] is not None:
                path.append({
                    'type': 'dataset',
                    'id': e[0]["dataset_id"],
                })
        except Exception:
            pass

        # Image is always present
        try:
            path.append({
                'type': 'image',
                'id': e[0]["image_id"]
            })
        except Exception:
            pass
        paths.append(path)
    return paths
